;------------------------------------------------------------------------
; HUB.ASM
;------------------------------------------------------------------------
    XPAGEON

; include related sub-modules
    INCLUDE "jmp_tbl.asm"               ; Rom jump table
    INCLUDE "hub_ep0.asm"               ; Hub endpoint 0 routines
    INCLUDE "resume.asm"                ; Port and Hub resume control routines
    INCLUDE "suspend.asm"               ; Hub suspend routines
    INCLUDE "port_pwr.asm"              ; Downstream power control routines

;------------------------------------------------------------------------
PORT_FROM_BITMAP:
; Finds and returns FIRST port with bit set in bitmap (from bit 0 to bit 6)
; Useful for bitmapped registers (REG_PORT_ENABLE, REG_PORT_SUSPEND, REG_PORT_RESUME)
; On entry A = port bitmap
; On exit  A = port number (0 to 6 [or MAX_PORTS-1]) or
;              0xFF if not found (or no bits set)
;          X = modified bitmap (found mask bit cleared)
;------------------------------------------------------------------------
    MOV     [TEMP], A                   ; save bitmap
    MOV     A, 0                        ; A = 0 (index into TABLE_PORT_MASK)
PFB_LOOP:
    PUSH    A                           ; save index
    INDEX   TABLE_PORT_MASK             ; get port bit mask
    AND     A, [TEMP]                   ; AND with input bitmap
    JNZ     PFB_FOUND                   ; if != 0 found the bit
    POP     A                           ; else get index
    INC     A                           ; bump index
    CMP     A, MAX_PORTS                ; last port?
    JZ      PFB_NOT_FOUND               ; if so, no bits set
    JMP     PFB_LOOP                    ; do again

PFB_FOUND:
    CPL     A                           ; A = port bit mask, complement
    AND     A, [TEMP]                   ; AND with input bitmap to clear the found bit
    MOV     X, A                        ; save new bitmap to X
    POP     A                           ; restore port number
    JMP     PFB_END                     ; done (return port in A, delta bitmap in X)
PFB_NOT_FOUND:
    MOV     A, FFh                      ; set port to 0xFF
PFB_END:
    RET

;------------------------------------------------------------------------
SET_PORT_CONTROL:
; On Entry: HUB_PORT_CTRL = port number to set
;           HUB_PORT_ACTION = forcing action (not forcing,force J/K/SE0)
; Then sets REG_PORT_CTRL_LOW or REG_PORT_CTRL_UP to appropriate forcing
; action
;------------------------------------------------------------------------
    MOV     A, [HUB_PORT_CTRL]
    CMP     A, 4
    JC      SPC_LOWER                   ; HUB_PORT_CTRL <  4
    JMP     SPC_UPPER                   ; HUB_PORT_CTRL >= 4
SPC_LOWER:
    CMP     A, 0                        ; if A = 0
    JZ      SPC_LOWER_SHIFT_DONE        ; shift left done
    PUSH    A                           ; save shift index
    MOV     A, [HUB_PORT_ACTION]        ; get forcing action
    ASL                                 ; shift 2 bits left
    ASL
    MOV     [HUB_PORT_ACTION], A        ; save back
    POP     A                           ; restore shift index
    DEC     A                           ; decrement
    JMP     SPC_LOWER                   ; do again
SPC_LOWER_SHIFT_DONE:                   ; done shifting
    MOV     A, [HUB_PORT_CTRL]          ; get port number
    INDEX   TABLE_PORT_CONTROL_MASK     ; get mask for the port
    MOV     [TEMP], A                   ; save to TEMP
    read    REG_PORT_CTRL_LOW           ; read control register
    AND     A, [TEMP]                   ; mask off desired bits
    OR      A, [HUB_PORT_ACTION]        ; set new control bits
    write_A REG_PORT_CTRL_LOW           ; set control register
    read    REG_PORT_CTRL_UP            ; read upper register
    NOP
    write_A REG_PORT_CTRL_UP            ; write back for second I/O write
                                        ; (Required for overriding HW on resume events)
    JMP     SET_PORT_CONTROL_EXIT

SPC_UPPER:                              ; port >= 4
    SUB     A, 4                        ; zero base the port number
    MOV     [TEMP1],A                   ; save modified port number to TEMP1
SPC_UPPER_LOOP:
    CMP     A, 0                        ; if A = 0
    JZ      SPC_UPPER_SHIFT_DONE        ; shift left done
    PUSH    A                           ; save shift index
    MOV     A, [HUB_PORT_ACTION]        ; get forcing action
    ASL                                 ; shift 2 bits left
    ASL
    MOV     [HUB_PORT_ACTION], A        ; save back
    POP     A                           ; restore shift index
    DEC     A                           ; decrement
    JMP     SPC_UPPER_LOOP              ; do again
SPC_UPPER_SHIFT_DONE:                   ; done shifting
    MOV     A, [TEMP1]                  ; get port number
    INDEX   TABLE_PORT_CONTROL_MASK     ; get mask for the port
    MOV     [TEMP], A                   ; save to TEMP
    read    REG_PORT_CTRL_UP            ; read control register
    AND     A, [TEMP]                   ; mask off desired bits
    OR      A, [HUB_PORT_ACTION]        ; set new control bits
    write_A REG_PORT_CTRL_UP            ; set control register
    read    REG_PORT_CTRL_LOW           ; read lower register
    NOP
    write_A REG_PORT_CTRL_LOW           ; write back for second I/O write
                                        ; (Required for overriding HW on resume events)
SET_PORT_CONTROL_EXIT:
    RET

;------------------------------------------------------------------------
SET_PORT_CONNECT:
; On entry A = port number (0-3)
;          X = HUB_wPortStatusX_LSB
; Sets connect bit in port status and port change
; Sets port state to DISABLED
;------------------------------------------------------------------------
    PUSH    A                           ; save port number
    MOV     A, [X+0]                    ; get port status
    OR      A, MASK_PORT_CONNECTION     ; set connect bit
    MOV     [X+0], A                    ; and save
    MOV     A, [X+2]                    ; get port change
    OR      A, MASK_PORT_CONNECTION     ; set change bit
    MOV     [X+2], A                    ; and save
    POP     A                           ; restore port number
    ADD     A, HUB_PORT_STATE_BASE      ; add state base
    SWAP    A, X                        ; set X = address to state
    MOV     A, PORT_DISABLED            ; A = DISABLED
    MOV     [X+0], A                    ; bump port state to DISABLED
    RET

;------------------------------------------------------------------------
SET_PORT_DISCONNECT:
; On entry A = port number (0-3)
;          X = HUB_wPortStatusX_LSB
; Clears connect bit and low speed bit in port status
; Sets connect change bit and clears reset change bit in port change
; Sets port state to DISCONNECTED
;------------------------------------------------------------------------
    PUSH    A                           ; save port
    PUSH    X                           ; save status offset
; Clear enable bit in REG_PORT_ENABLE
    INDEX   TABLE_PORT_MASK             ; get enable bit mask
    CPL                                 ; complement for disable
    MOV     [TEMP], A                   ; save to TEMP
    read    REG_PORT_ENABLE             ; get port enable register contents
    AND     A, [TEMP]                   ; clear port enable bit
    write_A REG_PORT_ENABLE             ; save back to port enable register
    MOV     [HUB_LAST_PORT_ENABLE], A   ; save to last port enable
; Clear the status bit and set the change bit
    POP     X                           ; restore status offset, keep A (port) on stack
    MOV     A, [X+0]                    ; get port status LSB
    AND     A, ~MASK_PORT_CONNECTION    ; clear PORT_CONNECTION bit
    AND     A, ~MASK_PORT_ENABLE        ; clear PORT_ENABLE bit
	AND	A,  ~MASK_PORT_SUSPEND
    MOV     [X+0], A                    ; and save
    MOV     A, [X+1]                    ; get port status MSB
    AND     A, ~MASK_PORT_LOW_SPEED     ; clear PORT_LOW_SPEED status bit
    MOV     [X+1], A                    ; and save
    MOV     A, [X+2]                    ; get port change LSB
    OR      A, MASK_PORT_CONNECTION     ; set C_PORT_CONNECTION bit
    AND     A, ~MASK_PORT_RESET         ; clear C_PORT_RESET bit
    AND	    A,  ~MASK_PORT_SUSPEND          ; clear C_MASK_PORT_SUSPEND
    MOV     [X+2], A                    ; and save to port change LSB
    POP     A                           ; restore port number
    ADD     A, HUB_PORT_STATE_BASE      ; add state base
    SWAP    A, X                        ; set X = address to state
    MOV     A, PORT_DISCONNECTED        ; A = DISCONNECTED
    MOV     [X+0], A                    ; bump state to DISCONNECTED
    RET

;------------------------------------------------------------------------
SET_PORT_DISABLE:
; On entry A = port number (0-3)
;          X = HUB_wPortStatusX_LSB
; Clears enabled bit only in port status
; Sets enable change bit in port change
; Sets port state to DISABLED
;------------------------------------------------------------------------
    PUSH    A                           ; save port
    PUSH    X                           ; save status offset
; Clear enable bit in REG_PORT_ENABLE
    INDEX   TABLE_PORT_MASK             ; get enable bit mask
    CPL                                 ; complement for disable
    MOV     [TEMP], A                   ; save to TEMP
    read    REG_PORT_ENABLE             ; get port enable register contents
    AND     A, [TEMP]                   ; clear port enable bit
    write_A REG_PORT_ENABLE             ; save back to port enable register
    MOV     [HUB_LAST_PORT_ENABLE], A   ; save to last port enable
; Clear the status bit and set the change bit
    POP     X                           ; restore status offset, keep A (port) on stack
    MOV     A, [X+0]                    ; get port status LSB
    AND     A, ~MASK_PORT_ENABLE        ; clear PORT_ENABLE bit
	AND	    A,  ~MASK_PORT_SUSPEND      ; clear PORT_SUSPEND bit
    MOV     [X+0], A                    ; and save
    MOV     A, [X+2]                    ; get port change LSB
    OR      A, MASK_PORT_ENABLE			; set C_PORT_ENABLE bit
    AND	    A,  ~MASK_PORT_SUSPEND      ; clear C_PORT_SUSPEND
    MOV     [X+2], A                    ; and save to port change LSB
    POP     A                           ; restore port number
    ADD     A, HUB_PORT_STATE_BASE      ; add state base
    SWAP    A, X                        ; set X = address to state
    MOV     A, PORT_DISABLED            ; A = PORT_DISABLED
    MOV     [X+0], A                    ; bump state to DISCONNECTED
    RET

;------------------------------------------------------------------------
SET_LOW_SPEED:
; On entry X = HUB_wPortStatusX_MSB
; Sets low speed bit in port status
;------------------------------------------------------------------------
    MOV     A, [X+0]                    ; yes
    OR      A, MASK_PORT_LOW_SPEED      ; set low speed status bit
    MOV     [X+0], A                    ; and save
    RET

;------------------------------------------------------------------------
TEST_STATUS_CHANGE:
; Test for any bits set in wHubChange or wPort{x}Change_LSB and set appropriate
; bits in BITMAP_HUB_EP1
; Load fifo and set REG_EP_B1_MODE to ACK if change is pending.
; Measured execution time (29us)
;------------------------------------------------------------------------
    MOV     A, 0                        ; A = 0
    MOV     [TEMP], A                   ; use TEMP as temporary BITMAP_HUB_EP1
    MOV     [TEMP1], A                  ; use TEMP1 as port number
    MOV     A, [HUB_wHubChange_LSB]     ; test for hub change
    CMP     A, 0                        ; if 0, no change
    JZ      TSC_NO_HUB_CHANGE
    MOV     A, MASK_HUB_CHANGE          ; else set hub change bit
    MOV     [TEMP], A                   ; save to TEMP
TSC_NO_HUB_CHANGE:
    MOV     X, HUB_PORT_STATUS_BASE+2   ; point X to wPortChange0_LSB
TSC_LOOP:
    MOV     A, [X+0]                    ; get wPortChange
    CMP     A, 0                        ; if 0, no change
    JZ      TSC_NO_CHANGE
    MOV     A, [TEMP1]                  ; else get port number
    INDEX   TABLE_PORT_CHANGE           ; get bitmask for the port
    OR      A, [TEMP]                   ; OR with pending bits
    MOV     [TEMP], A                   ; save back to TEMP
TSC_NO_CHANGE:
    SWAP    A, X                        ; A = X
    ADD     A, 4                        ; point A to next wPortChange
    SWAP    A, X                        ; X = A
    INC     [TEMP1]                     ; bump port count
    MOV     A, [TEMP1]                  ; A = port count
    CMP     A, MAX_PORTS                ; A = MAX_PORTS?
    JC      TSC_LOOP                    ; NO, test next port

    MOV     A, [TEMP]                   ; A = new bitmap
    MOV     [BITMAP_HUB_EP1], A         ; save to bitmap
    CMP     A, 0                        ; Change pending?
    JZ      TSC_NO_DATA                 ; no
    MOV     [FIFO_EP_B1], A             ; else load into fifo
    unlock_EP  REG_EP_B1_MODE, ACK_IN   ; Next INT should ACK the data
    JMP     TSC_WITH_DATA
TSC_NO_DATA:
    unlock_EP  REG_EP_B1_MODE, NAK_IN   ; No data so NAK the INT
TSC_WITH_DATA:
    RET

;------------------------------------------------------------------------
TASK_PORT_CHANGE:
; Hub processor (main loop task)
; MUST re-enable HUB interrupt upon completion
;------------------------------------------------------------------------
    MOV     A, [SHADOW_CONNECT_STAT]
    XOR     A, [HUB_LAST_CONNECT_STAT]  ; XOR and save only bits that changed
    CMP     A, 0                        ; did bits change?
    JZ      TASK_PORT_CHANGE_END        ; no port change!

TPC_CHANGE_DETECT:                      ; yes, change detected!
    MOV     [HUB_CONNECT_CHANGE], A     ; save change bitmap to HUB_CONNECT_CHANGE
    MOV     A, [SHADOW_CONNECT_STAT]    ; get current connect status
    MOV     [HUB_LAST_CONNECT_STAT], A  ; save to last connect status

TPC_LOOP:
    MOV     A, [HUB_CONNECT_CHANGE]     ; get only new bits
    CMP     A, 0                        ; done?
    JZ      TASK_PORT_CHANGE_END        ; yes
    CALL    PORT_FROM_BITMAP            ; no, get port
    SWAP    A,X                         ; A = modified bitmap
    MOV     [HUB_CONNECT_CHANGE],A     ; save back to connenct change
    SWAP    A,X                         ;get port back in A
    MOV     [TEMP], A                   ; save port to TEMP
    MOV     A,[TEMP]                    ;else get port back in A
    INDEX   TABLE_PORT_MASK             ; get port bit mask
    MOV     [TEMP1], A                  ; save bit mask to TEMP1
    MOV     A, [SHADOW_CONNECT_STAT]    ; get latest connect status
    AND     A, [TEMP1]                  ; AND with port bit mask
    JZ      TPC_DISCONNECT              ; if 0, disconnect detected
TPC_CONNECT:                            ; else connect detect
    MOV     A, [TEMP]                   ; A = port number
    PUSH    A                           ; save port number
    ASL                                 ; Mult by 4
    ASL
    ADD     A, HUB_PORT_STATUS_BASE     ; add offset to port status base
    MOV     [TEMP2], A                  ; save offset to TEMP2
    MOV     X, A                        ; X = index to wPortStatus_LSB
    POP     A                           ; restore port number
    CALL    SET_PORT_CONNECT            ; set port connect bits, set port state
    MOV     A, [SHADOW_PORT_SPEED]      ; get speed register
    AND     A, [TEMP1]                  ; AND with port bit mask, low speed?
    JZ      TPC_LOOP                    ; no, do it again
    MOV     A, [TEMP2]                  ; get offset to wPortStatus_LSB
    INC     A                           ; bump to MSB
    MOV     X, A                        ; X = index to wPortStatus_MSB
    CALL    SET_LOW_SPEED               ; set low speed bits
    JMP     TPC_LOOP                    ; do it again

TPC_DISCONNECT:                         ; disconnect detected
    MOV     A, [TEMP]                   ; A = port number
    PUSH    A                           ; save port number
    ASL                                 ; Mult by 4
    ASL
    ADD     A, HUB_PORT_STATUS_BASE     ; add offset to port status base
    MOV     X, A                        ; X = index to wPortStatus_LSB
    POP     A                           ; restore port number
    CALL    SET_PORT_DISCONNECT         ; set port disconnect bits, set port state

	MOV	A,1
	MOV     [STATE_RESUME_PENDING], A 
	clr_flag    FLAG_PORT_RESUME
	     
    JMP     TPC_LOOP                    ; do it again




TASK_PORT_CHANGE_END:
    clr_flag    FLAG_PORT_CHANGE        ; clear the flag
   
    enable_hub_int                      ; reenable the interrupt
    RET

;------------------------------------------------------------------------
TASK_PORT_RESET:
; USB Downstream Port Reset control task (main loop task)
; This task is initiated by SET_PORT_RESET (HUB_EP0C.ASM).
; This task handles the timeout for SE0 assertion and the removal of SE0
; control, and then places the port in the enabled state.
; wPortStatus and wPortChange are handled by both routines.
;------------------------------------------------------------------------
    MOV     A, [HUB_PORT_RESET_COUNT]   ; get timer incremented counter
    CMP     A, PORT_RESET_LIMIT         ; expired?
    JC      TASK_PORT_RESET_END         ; no, get out

; reset timeout expired, remove SE0 signaling and set corresponding wPortStatus wPortChange bits
    MOV     A, [HUB_RESET_PORT]         ; get hub reset port
    PUSH    A                           ; save port number

    MOV     [HUB_PORT_CTRL], A          ; save port to HUB_PORT_CTRL
    MOV     A, MASK_NOT_FORCING         ; remove forced SE0 state
    MOV     [HUB_PORT_ACTION], A        ; save forcing action
    CALL    SET_PORT_CONTROL            ; set action to appropriate port

    POP     A                           ; restore port number
    SWAP    A, X                        ; use X as offset into state base
    MOV     A, PORT_ENABLED             ; A = state of selected port
    MOV     [X+HUB_PORT_STATE_BASE], A

    SWAP    A, X                        ; A = port number
    ASL                                 ; A*2
    ASL                                 ; A*4
    SWAP    A, X                        ; X = port*4 (offset into wPortStatus base)
    MOV     A, [X+HUB_PORT_STATUS_BASE] ; get wPortStatus LSB
    AND     A, ~MASK_PORT_RESET         ; clr port reset status bit
    AND     A, ~MASK_PORT_SUSPEND       ; clr port suspend bit
    OR      A, MASK_PORT_ENABLE         ; set the enable bit
    MOV     [X+HUB_PORT_STATUS_BASE], A ; save wPortStatus LSB

    INC     X                           ; X+=2, point to wPortChange LSB
    INC     X
    MOV     A, [X+HUB_PORT_STATUS_BASE] ; get wPortChange
    OR      A, MASK_PORT_RESET          ; set port reset change bit
    MOV     [X+HUB_PORT_STATUS_BASE], A ; save change

; set enable bit in REG_PORT_ENABLE
    MOV     A, [HUB_RESET_PORT]
    INDEX   TABLE_PORT_MASK
    MOV     [TEMP], A
    read    REG_PORT_ENABLE             ; get port enable register contents
    OR      A, [TEMP]                   ; set port enable bit
    write_A REG_PORT_ENABLE             ; save back to port enable register
    MOV     [HUB_LAST_PORT_ENABLE], A   ; save to last port enable
    clr_flag    FLAG_PORT_RESET         ; clear the main loop flag to discontinue processing
TASK_PORT_RESET_END:
    RET

;------------------------------------------------------------------------
TASK_PORT_DISABLED:
; USB Downstream Port Disabled by Hub HW control task (main loop task)
; This task is initiated by INT_HUB (VECTOR.ASM).
; This task handles the Hub HW disabling of a DS port due to babble detect.
; wPortStatus and wPortChange are handled by this routine.
;------------------------------------------------------------------------
    read    REG_PORT_ENABLE             ; read enable register
    PUSH    A                           ; save
    XOR     A, [HUB_LAST_PORT_ENABLE]   ; XOR with last enable
    MOV     [HUB_ENABLE_CHANGE], A      ; save to enable change
    POP     A                           ; restore enable register
    MOV     [HUB_LAST_PORT_ENABLE], A   ; save to last port enable

TPD_LOOP:
    MOV     A, [HUB_ENABLE_CHANGE]      ; get only disabled bits
    CMP     A, 0                        ; done?
    JZ      TASK_PORT_DISABLED_END      ; yes
    CALL    PORT_FROM_BITMAP            ; no, get port
    MOV     [TEMP], A                   ; save port to TEMP
    SWAP    A, X                        ; A = modified bitmap
    MOV     [HUB_ENABLE_CHANGE], A      ; save back to enable change
    SWAP    A, X                        ; A = port
    INDEX   TABLE_PORT_MASK             ; get port bit mask
    MOV     [TEMP1], A                  ; save bit mask to TEMP1

    MOV     A, [TEMP]                   ; A = port number
    PUSH    A                           ; save port number
    ASL                                 ; Mult by 4
    ASL
    ADD     A, HUB_PORT_STATUS_BASE     ; add offset to port status base
    MOV     X, A                        ; X = index to wPortStatus_LSB
    POP     A                           ; restore port number
;tpmV3.2    CALL    SET_PORT_DISCONNECT         ; set port disconnect bits, set port state
    CALL    SET_PORT_DISABLE			;tpmV3.2 set port disable bits only (11.8.1).
										;tpmV3.2 host may try to reset port.
    JMP     TPD_LOOP                    ; do it again

TASK_PORT_DISABLED_END:
    clr_flag    FLAG_PORT_DISABLED
    RET

;------------------------------------------------------------------------
GET_DESCRIPTOR_LENGTH:    ;*? make note on how host lies! (r.e. Gary's code)
;------------------------------------------------------------------------
	MOV     A, [HUB_wLength_LSB] 	    ; load requested transfer length
	CMP     A, 0                        ; test for length > 256 bytes
	JZ      USE_ACTUAL_LENGTH           ; nothing should be that large
    CMP     A, [HUB_DATA_COUNT]         ; compared to the amount of data
	JNC     USE_ACTUAL_LENGTH
	MOV     [HUB_DATA_COUNT], A         ; use requested length
USE_ACTUAL_LENGTH:
    RET                                 ; return

;------------------------------------------------------------------------
SEND_BUFFER:
;------------------------------------------------------------------------
    unlock  REG_EP_B0_MODE, NAK_IN_OUT
	PUSH    X                           ; save X on stack
	MOV     A, 0                        ; load count into A
SEND_BUFFER_LOOP:
	SWAP    A, X                        ; move count into X
	MOV     A, [HUB_DATA_START]         ; load ROM index into A
	INDEX   TABLE_DESCRIPTOR            ; get the ROM byte
    MOV     [X + FIFO_EP_B0], A         ; save the byte in FIFO
    INC     [HUB_DATA_START]            ; bump ROM index
    INC     X                           ; bump the count in X
	SWAP    A, X                        ; move count to A
	CMP     A, FIFO_SIZE                ; compare count to 8
	JNZ     SEND_BUFFER_LOOP            ; keep copying

    MOV     [HUB_BYTE_COUNT], A         ; default to 8-bytes
	MOV     A, [HUB_DATA_COUNT]	        ; update byte counter
    SUB     A, FIFO_SIZE	            ; subtract 8-bytes sent
	JNC     SEND_BUFFER_END
    MOV     A, [HUB_DATA_COUNT]         ; update the byte count in memory
    MOV     [HUB_BYTE_COUNT], A
    MOV     A, 0                        ; done

SEND_BUFFER_END:
	MOV     [HUB_DATA_COUNT], A         ; update byte counter
	POP     X                           ; restore X from stack
	RET

;------------------------------------------------------------------------
LOAD_STATUS:
; On entry: HUB_DATA_START points to status data (Hub or Port)
;           HUB_DATA_COUNT is the number of chars to send
; This routine loads HUB_DATA_COUNT characters to FIFO_EP_B0
; for the hub or the requested port (see GET_PORT_STATUS, GET_HUB_STATUS [HUB_EP0C.ASM]).
;------------------------------------------------------------------------
    ;following line strictly not necessary but makes CY3652 more reliable
    ;bth 2/10/99
    unlock  REG_EP_B0_MODE, NAK_IN_OUT
    MOV     X, 0                        ; X = 0, clear count
	PUSH    X                           ; save to stack
	MOV     X, [HUB_DATA_START]         ; point X to data
LS_LOOP:
    MOV     A, [X+0]                    ; get status/change data
    INC     [HUB_DATA_START]            ; bump data pointer
    POP     X                           ; restore count
    MOV     [X + FIFO_EP_B0], A         ; save the byte in FIFO
    INC     X                           ; bump the count
	SWAP    A, X                        ; move count to A
	PUSH    A                           ; save count to stack
	CMP     A, [HUB_DATA_COUNT]         ; compare count to COUNT
    JNC     LS_LOOP_END                 ; exit if done
	MOV     X, [HUB_DATA_START]         ; else keep copying
    JMP     LS_LOOP

LS_LOOP_END:
    POP     X                           ; clean up stack
    RET

XPAGEOFF
; SET_PORT_CONTROL mask lookup table for REG_PORT_CTRL_LOW and REG_PORT_CTRL_UP
TABLE_PORT_CONTROL_MASK:
    DB      11111100b       ; control mask for ports 0 and 5
    DB      11110011b       ;                        1     6
    DB      11001111b       ;                        2     7
    DB      00111111b       ;                        3

TABLE_LS_EOP_START:
    DB      00000011b       ; force SE0 ports 0 or 5
    DB      00001100b       ;                 1    6
    DB      00110000b       ;                 2    7
    DB      11000000b       ;                 3

TABLE_LS_EOP_END:
; Full-speed J signaling
    DB      00000001b       ; force Diff 1 ports 0 or 5 (AND to mask off SE0 to J)
    DB      00000100b       ;                    1    6
    DB      00010000b       ;                    2    7
    DB      01000000b       ;                    3
; Low-speed J signaling
    DB      00000010b       ; force Diff 0 ports 0 or 5 (AND to mask off SE0 to J)
    DB      00001000b       ;                    1    6
    DB      00100000b       ;                    2    7
    DB      10000000b       ;                    3

TABLE_PORT_CHANGE:
    DB      MASK_PORT1_CHANGE
    DB      MASK_PORT2_CHANGE
    DB      MASK_PORT3_CHANGE
    DB      MASK_PORT4_CHANGE
    DB      MASK_PORT5_CHANGE
    DB      MASK_PORT6_CHANGE
    DB      MASK_PORT7_CHANGE

    XPAGEON
; END HUB.ASM ---------------------------------------------------------