;------------------------------------------------------------------------
; HUB.INC
;------------------------------------------------------------------------
; INCLUDE file for HUB.ASM
;------------------------------------------------------------------------

INVALID_TYPE:           EQU FFh     ; invalid bmRequestType (table delimiter)
INVALID_REQUEST:        EQU FFh     ; invalid bRequest      (table delimiter)
INVALID_FEATURE:        EQU FFh     ; invalid feature selector

;------------------------------------------------------------------------
; SETUP PACKET definitions
;------------------------------------------------------------------------
; Standard Device Requests (default pipe, EP0)
; protocol --> [bmRequestType][bRequest][wValue][wIndex][wLength][..Data..]
;------------------------------------------------------------------------
; [bmRequestType]
; Standard Request Types ([1] pg 174)
;------------------------------------------------------------------------
; Host to Device                -- applicable bRequest codes --
ST_IN_0:                EQU 00h     ; Device specific
                                    ; CLEAR_FEATURE, SET_ADDRESS, SET_CONFIGURATION
                                    ; SET_DESCRIPTOR, SET_FEATURE
ST_IN_1:                EQU 01h     ; Interface specific
                                    ; CLEAR_FEATURE, SET_FEATURE, SET_INTERFACE
ST_IN_2:                EQU 02h     ; Endpoint specific
                                    ; CLEAR_FEATURE, SET_FEATURE
; Device to Host
ST_OUT_0:               EQU 80h     ; Device specific
                                    ; GET_CONFIGURATION, GET_DESCRIPTOR, GET_STATUS
ST_OUT_1:               EQU 81h     ; Interface specific
                                    ; GET_INTERFACE, GET_STATUS
ST_OUT_2:               EQU 82h     ; Endpoint specific
                                    ; GET_STATUS, SYNC_FRAME
;------------------------------------------------------------------------
; [bRequest]
; Standard Requests  ([1] pg 175)
; Hub Class Requests ([2] pg 262)
;------------------------------------------------------------------------
GET_STATUS:             EQU 00h     ; STD and Hub
CLEAR_FEATURE:          EQU 01h     ; STD and Hub
GET_STATE:              EQU 02h     ;         Hub only (optional for hub's)
SET_FEATURE:            EQU 03h     ; STD and Hub
SET_ADDRESS:            EQU 05h     ; STD         only
GET_DESCRIPTOR:         EQU 06h     ; STD and Hub
SET_DESCRIPTOR:         EQU 07h     ; STD and Hub
GET_CONFIGURATION:      EQU 08h     ; STD         only
SET_CONFIGURATION:      EQU 09h     ; STD         only
GET_INTERFACE:          EQU 0Ah     ; STD         only (optional for hub's)
SET_INTERFACE:          EQU 0Bh     ; STD         only (optional for hub's)
SYNCH_FRAME:            EQU 0Ch     ; STD         only (optional for hub's)

;------------------------------------------------------------------------
; [wValue]
; Descriptor Types ([1] pg 175)
;------------------------------------------------------------------------
DT_DEVICE:              EQU 01h
DT_CONFIGURATION:       EQU 02h
DT_STRING:              EQU 03h
DT_INTERFACE:           EQU 04h
DT_ENDPOINT:            EQU 05h
DT_HUB:                 EQU 29h     ; (always 0x29 for Hub Descriptor)
                                    ; [USB spec Rev. 1.1 , Table 11-8]
STRING_LANGUAGE:        EQU   0
STRING_MANUFACTURER:    EQU   1
STRING_PRODUCT:         EQU   2
STRING_SERIAL:          EQU   3
STRING_CONFIGURATION:   EQU   4
STRING_INTERFACE:       EQU   5 

HUB_CLASSCODE:              EQU 09h

;------------------------------------------------------------------------
; [wValue]
; Feature Selectors ([1] pg 175)
;------------------------------------------------------------------------
ENDPOINT_STALL:         EQU 00h     ; endpoint recipient
DEVICE_REMOTE_WAKEUP:   EQU 01h     ; device recipient
; [wIndex]LSB mask for endpoint
MASK_ENDPOINT:          EQU 00000001b

;------------------------------------------------------------------------
; Hub-Class Specific Requests (default pipe, EP0)
; protocol --> [bmRequestType][bRequest][wValue][wIndex][wLength][..Data..]
;------------------------------------------------------------------------
; [bmRequestType]
; Hub-Specific Request Types ([2] Table 11-10, pg 261)
;------------------------------------------------------------------------
; Host to Hub                       -- applicable Hub Class Requests (bRequest) --
; ClearHubFeature, SetHubDescriptor and SetHubFeature
HT_IN_0:                EQU 20h     ;
                                    ; CLEAR_FEATURE,  SET_DESCRIPTOR, SET_FEATURE
; ClearPortFeature and SetPortFeature
HT_IN_1:                EQU 23h     ; CLEAR_FEATURE,  SET_FEATURE
; Hub to Host
; GetHubDescriptor and GetHubStatus
HT_OUT_0:               EQU A0h     ; GET_DESCRIPTOR, GET_STATUS
; GetBusState and GetPortStatus
HT_OUT_1:               EQU A3h     ; GET_STATE,      GET_STATUS

;------------------------------------------------------------------------
; Hub Class Feature Selectors ([2] Table 11-12)
; [wValue]
; Hub features
;------------------------------------------------------------------------
C_HUB_LOCAL_POWER:      EQU 0
C_HUB_OVER_CURRENT:     EQU 1
;------------------------------------------------------------------------
; Port features
PORT_CONNECTION:        EQU 0
PORT_ENABLE:            EQU 1
PORT_SUSPEND:           EQU 2
PORT_OVER_CURRENT:      EQU 3
PORT_RESET:             EQU 4
PORT_POWER:             EQU 8
PORT_LOW_SPEED:         EQU 9
C_PORT_CONNECTION:      EQU 16
C_PORT_ENABLE:          EQU 17
C_PORT_SUSPEND:         EQU 18
C_PORT_OVER_CURRENT:    EQU 19
C_PORT_RESET:           EQU 20
;------------------------------------------------------------------------

;------------------------------------------------------------------------
; Descriptor Equates
OFFSET_CONFIG_ATTRIBUTES:   EQU 7          ; offset to bmAttributes (config)
OFFSET_PORT_POWER_MASK:     EQU 8          ; offset into HUB_CLASS_DESCRIPTOR to PortPwrCtrlMask
MASK_REMOTE_WAKEUP:         EQU 00100000b  ; bmAttributes bitmasks
MASK_SELF_POWERED:          EQU 01000000b
MASK_BUS_POWERED:           EQU 10000000b