;------------------------------------------------------------------------
; HUBDESC.ASM
; ROM Tables (system descriptors)   hub
;------------------------------------------------------------------------
    XPAGEOFF
;------------------------------------------------------------------------
; Standard USB Descriptor
; [1] 9.6.1 Device
; [2] 11.11.1 Standard Descriptors
; (LSB, MSB) Little-Endian format (MSB at the higher memory location)

TABLE_DESCRIPTOR:
HUB_DEVICE_DESCRIPTOR:  ; (* denotes pre-defined fields in STD device descriptor, USB Rev 1.1)
    DB  18                  ;  bLength
    DB  DT_DEVICE           ;  bDescriptorType
    DB  10h, 01h            ;  bcdUSB  (USB 1.10)
    DB  09h                 ;  bDeviceClass (HUB_CLASSCODE)
    DB  01h                 ;  bDeviceSubClass
    DB  00h                 ;   bDeviceProtocol
    DB  08h                 ;* bMaxPacketSize        (wMaxPacketSize in RC2 Rev1.1 ???)
    DB  25h,09h             ;  idVendor (Lakeview)
    DB  35h,12h             ;  idProduct (1235)
    DB  01h,00h             ;  bcdDevice
    DB  01h                 ;  iManufacturer
    DB  02h                 ;  iProduct
    DB  00h                 ;  iSerialNumber
    DB  01h                 ;  bNumConfigurations

; [1] 9.6.2 Configuration
; [2] 11.11.1 Standard Descriptors
HUB_CONFIG_DESCRIPTOR:
    DB  9                   ;  bLength
    DB  DT_CONFIGURATION    ;  bDesriptorType
    DB  19h,00h             ;?  bTotalLength (combined length of all descriptors)
    DB  01h                 ;?  bNumInterfaces
    DB  01h                 ;?  bConfigurationValue
    DB  00h                 ;?  iConfiguration
    DB (80H | MASK_SELF_POWERED | MASK_REMOTE_WAKEUP)   ;bmAttributes
                            ;  bmAttributes
                            ;       11100000
                            ;       ||||||||___ (reserved)
                            ;       |||________ Remote Wakeup (1 = enabled)
                            ;       ||_________ Self Powered  (1 = enabled)
                            ;       |__________ Reserved Bit, must be a 1


    DB  49                  ;* MaxPower (in 2ma units, 50 = 100ma)

; [1] 9.6.3 Interface
; [2] 11.11.1 Standard Descriptors
HUB_INTERFACE_DESCRIPTOR:
    DB  9                   ;  bLength
    DB  DT_INTERFACE        ;  bDescriptorType
    DB  00h                 ;  bInterfaceNumber
    DB  00h                 ;  bAlternateSetting
    DB  1                   ;* bNumEndpoints         (always 1)
    DB  09h                 ;* bInterfaceClass       (HUB_CLASSCODE)
    DB  01h                 ;* bInterfaceSubClass
    DB  00h                 ;* bInterfaceProtocol
    DB  00h                 ;  iInterface

; [1] 9.6.4 Endpoint
; [2] 11.11.1 Standard Descriptors
HUB_ENDPOINT_DESCRIPTOR:    ; (for Status Change Endpoint (EP1), required for hub's)
    DB  7                   ;  bLength
    DB  DT_ENDPOINT         ;  bDescriptorType
    DB  10000001b           ;* bEndpointAddress
       ;||||||||______________ address 1
       ;|_____________________ Bit 7=dir, IN (1, xfers to host)
    DB  00000011b           ; * bmAttributes(Interrupt, always an IN transaction)
    DB  01h,00h             ; * wMaxPacketSize  (one char, BITMAP_HUB_EP1 bitmap)
    DB  FFh                 ;* bInterval        (MAX allowable interval)
HUB_CONFIG_DESCRIPTOR_END:

; Hub Class-specific Descriptor
; [2] 11.11.2.1 Hub Descriptor
HUB_CLASS_DESCRIPTOR:
    DB  9                   ; bDescLength
    DB  DT_HUB              ; bDescriptorType
    DB  MAX_PORTS           ; bNbrPorts (See EQUATE.INC)


    DB  00001001b, 00h      ; wHubCharacteristics
       ;xxx|||||___________ 01=individual port power switching
       ;   |||_____________  0=hub is not p/o a compound device
       ;   ||______________ 01=over-current protection (bus-powered)
                                        ; ready for enumeration
    DB  0Ah                 ; bPwrOn2PwrGood        (time in 2ms intervals, 20ms, port power-on delay)
    DB  49                  ; bHubContCurrent       (default HUB current req. to 100ma )
    DB  0                   ; DeviceRemovable       (usually variable, fixed in this application)
                            ;           Assume ALL 4 downstream ports removable (bitmaps = 0)

    DB  11111111b           ; PortPwrCtrlMask       (all ports must rx manual commands to turn
                            ;           power on/off (SetPortFeature(POWER_PORT))
HUB_CLASS_DESCRIPTOR_END:

TABLE_STRING:
HUB_LANGUAGE_STRING:
    db 04h          ; Length
    db 03h          ; Type (3=string)
    db 09h          ; Language:  English
    db 04h          ; Sub-language: Default
HUB_LANGUAGE_STRING_END:

HUB_MANUFACTURER_STRING:
    db  HUB_MANUFACTURER_STRING_END - HUB_MANUFACTURER_STRING       ; Length
    db 03h          ; Type (3=string)
    dsu "USB Complete" ;
HUB_MANUFACTURER_STRING_END:
HUB_PRODUCT_STRING:
    db HUB_PRODUCT_STRING_END - HUB_PRODUCT_STRING	; Length
    db 03h          ; Type (3=string)
    dsu "Hub HID Example" ;
HUB_PRODUCT_STRING_END:
TABLE_STRING_END:

   XPAGEON