;===========================================================================
; File Name: MAIN.ASM
;
; Initialization code and  Main processing loop code
;===========================================================================

;------------------------------------------------------------------------
INITIALIZE:
; Preset system parameters
;------------------------------------------------------------------------
   MOV     A, ffh           ; config Ports to resistive mode
    write_A REG_GPIO_CONFIG
    MOV     A, 0                        ; set A = 0
    write_A REG_PORT_0_INT_ENABLE
    write_A REG_PORT_1_INT_ENABLE
    write_A REG_PORT_2_INT_ENABLE
    write_A REG_PORT_3_INT_ENABLE
    write_A REG_DAC_ISINK_4             ; Preset downstream port control DAC's to lowest current
    write_A REG_DAC_ISINK_5
    write_A REG_DAC_ISINK_6
    write_A REG_DAC_ISINK_7
    write_A REG_PORT_ENABLE             ; clear port enable bits
    write_A REG_PORT_CTRL_LOW           ; set all ports to not forcing
    write_A REG_PORT_CTRL_UP

    CALL    POWER_OFF_ALL               ; Power off all ports and reset port mem locs
    CALL    INIT_GPIO

    MOV     A,0
; clear all zero init mem locs
    MOV     [FLAG_PORT_CHANGE], A       ; clear system flags
    MOV     [FLAG_HUB_EP0], A
    MOV     [FLAG_PORT_RESET], A
    MOV     [FLAG_PORT_RESUME], A
    MOV     [FLAG_HUB_SUSPEND], A
    MOV     [FLAG_HUB_EP0_ACK], A
    MOV     [FLAG_REMOTE_WAKEUP], A
    MOV     [FLAG_STALL], A
    MOV     [FLAG_PORT_DISABLED], A
    MOV     [HUB_EP0_TOKEN], A          ; reset Hub received token
    MOV     [HUB_BYTE_COUNT], A         ; clear the send count
    MOV     [HUB_DATA_COUNT], A         ; clear the data counter
    MOV     [HUB_SUSPEND_COUNT], A      ; clear the suspend counter
    MOV     [SHADOW_PORT_SPEED], A
    MOV     [SHADOW_CONNECT_STAT], A    ; clear current port connect status
    MOV     [HUB_LAST_CONNECT_STAT], A  ; clear last port connect detect status
    MOV     [HUB_LAST_PORT_ENABLE], A   ; clear last port enable
    MOV     [HUB_CONNECT_CHANGE], A     ; clear the connect status change bits
    MOV     [HUB_ENABLE_CHANGE], A      ; clear the enable change bits (babble)
    MOV     [HUB_RESET_PORT], A         ; clear the reset port
    MOV     [HUB_RESUME_PORT], A        ; clear the resume port
    MOV     [BITMAP_HUB_EP1], A         ; clear hub port status bitmap
    MOV     [SHADOW_PORT_RESUME], A     ; clear the port resume bitmap
    MOV     [HUB_wHubStatus_LSB], A     ; clear wHubStatus
    MOV     [HUB_wHubStatus_MSB], A
    MOV     [HUB_wHubChange_LSB], A     ; clear wHubChange
    MOV     [HUB_wHubChange_MSB], A
    MOV     [HUB_CONFIG_LSB], A         ; clear the configuration
    MOV     [HUB_CONFIG_MSB], A
    MOV     [HUB_ASSIGNED_ADDRESS], A   ; clear the address



    MOV     [UNUSED_INTS], A            ; clear unused int bit masks
; preset all system states to default state (1)
    MOV     A, DEFAULT_STATE
    MOV     [STATE_HUB], A
    MOV     X,MAX_PORTS-1
.loop:
    MOV     [X+HUB_PORT_STATE_BASE],A       ; Set ALL downstream ports to powered-off state (1)
    DEC     X
    JNC     .loop

    MOV     [STATE_RESUME_PENDING], A

    MOV     A,0ffh
    CPL     A
    AND     A, MASK_OVERCURRENT_INPUTS  ; mask off all but OC detect bits
    MOV     [SHADOW_PORT_OC],A           ; save new sample

    enable_hub_address                      ; enable hub EP0 for addressablilty

; write CONFIG values to I/O locations (REG's)
    write   0ffh,REG_PORT_FORCE_LOW1_4
    MOV     A,MAX_EMBEDDED_PORTS
    CMP     A,5
    JC      SKIP_HIGH
    write   0ffh,REG_PORT_FORCE_LOW5_7
SKIP_HIGH:
    write   CONFIG_GLOBAL_INT, REG_GLOBAL_INT_ENABLE
    write   CONFIG_EP_INT, REG_EP_INT_ENABLE
    write   CONFIG_STAT_CTRL, REG_STAT_CTRL
    unlock  REG_EP_B0_MODE, DISABLED
    unlock_EP  REG_EP_B1_MODE, DISABLED
    unlock  REG_EP_B1_COUNT, MASK_DATATOGGLE ; preset data1 bit for data0 on first data
    CALL HID_INIT
    RET



;------------------------------------------------------------------------
POWER_ON_RESET:
; program flow starts here after a power on reset
;------------------------------------------------------------------------
    DI                                  ; disable interrupts
    MOV     A, DSP_PRESET               ; preset data stack pointer
    SWAP    A, DSP
    MOV     A,0
    MOV     PSP,A
    CALL    INITIALIZE
    EI
    JMP     MAIN
;------------------------------------------------------------------------
USB_BUS_RESET:
; program flow starts here after a USB reset
;------------------------------------------------------------------------

	MOV	A, 0
	IOWR	4BH
	IOWR	4CH
    DI                                  ; disable interrupts
    MOV     A, DSP_PRESET               ; preset data stack pointer
    SWAP    A, DSP
    MOV     A,0
    MOV     PSP,A
    CALL    INITIALIZE
    unlock  REG_EP_B0_MODE, STALL
    EI

;------------------------------------------------------------------------
MAIN:
; System initialization and idle loop
; TASK dispatcher --> test flags/states and execute tasks (SET FLAG = EXECUTE TASK)
;------------------------------------------------------------------------

;*************************************************** System Task Dispatcher *****
MAIN_LOOP:
    feed_dog                            ; keep the dog alive...no task (nor ALL tasks)
                                        ;     should ever take more than 8ms except
                                        ;     possibly TASK_HUB_SUSPEND where the
                                        ;     dog gets a treat.

   CALL    LED_CONTROL
   CALL    TEST_STATUS_CHANGE          ; ALWAYS test for any status changes

; Process interrupt based threads ---------------------------------------

    test_flag FLAG_PORT_CHANGE          ; hub port change flag set?
    JNZ     ML_1
    CALL    TASK_PORT_CHANGE            ; process port change thread
ML_1:
    test_flag FLAG_HUB_EP0              ; hub default pipe flag set?
    JNZ     ML_2
    CALL    TASK_HUB_EP0                ; process hub default pipe thread

; Process timer delay based threads -------------------------------------
ML_2:
    test_flag FLAG_HUB_SUSPEND          ; upstream bus idle detected?
    JNZ     ML_3
    CALL    TASK_HUB_SUSPEND            ; process hub suspend thread
ML_3:
    test_state STATE_RESUME_PENDING, RESUME_START; DS port initiated resume (awake hub)?
    JNZ     ML_4
    CALL    TASK_RESUME_PENDING         ; process port selective resume thread
ML_4:
    test_flag FLAG_PORT_RESUME          ; DS port resume sequence taking place?
    JNZ     ML_5
    CALL    TASK_PORT_RESUME            ; process host(or port)initiated resume thread
ML_5:
    test_flag FLAG_PORT_RESET           ; Downstream port in reset signaling state?
    JNZ     ML_6
    CALL    TASK_PORT_RESET             ; process port reset thread
ML_6:
    test_flag FLAG_PORT_DISABLED        ; Has a DS port been disabled by HW?
    JNZ     ML_7
    CALL    TASK_PORT_DISABLED          ; process port disabled thread
ML_7:
    CALL    TASK_OVERCURRENT            ; process overcurrent detect thread
;(HID) Keyboard threads --------------------------------------------
    test_state  STATE_HUB, HUB_CONFIGURED ; is hub configured?
    JNZ     ML_9
    CALL    TASK_HID                       ; yes
ML_9:
    JMP     MAIN_LOOP                   ; keep checking flags....

ROM_HUB_LOWER_END:
; END MAIN.ASM ----------------------------------------------------------
