;------------------------------------------------------------------------
; MALLOC.INC
;------------------------------------------------------------------------
; Global memory allocation listing/usage
;------------------------------------------------------------------------

PSP_PRESET:         EQU 00h ; Program Stack Pointer preset address
NESTED_INTS:        EQU 1   ; Number of Interrupts allowed to nest (NOT allowed, use flags/tasks)
                            ; Use of flags and tasks allows FIFO-like processing of INT's
                            ; instead of LIFO processing due to nesting.
                            ; Also reduces interrupt latency (global int disable time).
NESTED_CALLS:       EQU 10  ; MAX number of CALL's allowed to nest
PSP_SIZE:           EQU (NESTED_INTS + NESTED_CALLS) * 2  ; 2 bytes/INT, 2 bytes/CALL

; System variable space
; !!!! Allocate ALL module variable data here !!!!
BASE__DATA_START:     EQU (PSP_PRESET + PSP_SIZE + 1)
; HUB.ASM definitions
BASE_HUB:           EQU BASE__DATA_START
STATE_HUB:          EQU BASE_HUB+0  ; USB device state for the hub
HUB_ASSIGNED_ADDRESS:EQU BASE_HUB+1 ; address assigned by host (wValue)
HUB_CONFIG_LSB:     EQU BASE_HUB+3  ; configuration value assigned by host (wValue)
HUB_CONFIG_MSB:     EQU BASE_HUB+4
HUB_PORT_POWER:     EQU BASE_HUB+5  ; port power bit mask (4321xxxxb) to DAC port bits
FLAG_GPIO:          EQU BASE_HUB+6  ; copy of HUB_CLASS_DESCRIPTOR (PortPwrCtrlMask)
;---------------------------
HUB_PORT_STATE_BASE:EQU BASE_HUB+7  ; Base address of ALL downstream port states
STATE_HUB_PORT0:    EQU BASE_HUB+7  ; downstream port 0 state (!!!states MUST be contiguous!!!)
;allocate space for more ports as specified by MAX_PORTS
HUB_PORT_STATE_END: EQU HUB_PORT_STATE_BASE+MAX_PORTS ; End address of ALL downstream port states
;---------------------------
BASE_HUB0:          EQU HUB_PORT_STATE_END
FLAG_HUB_EP0:       EQU BASE_HUB0+0 ; Main Loop flag for Hub endpoint 0 interrupts
FLAG_HUB_EP0_ACK:   EQU BASE_HUB0+1 ; Flag that HUB EP0 received an ACK
FLAG_HUB_SUSPEND:   EQU BASE_HUB0+2 ; Main loop flag for hub suspend
FLAG_PORT_CHANGE:   EQU BASE_HUB0+3 ; Main Loop flag for Hub port change interrupts
FLAG_PORT_RESET:    EQU BASE_HUB0+4 ; Main loop flag for port reset control (host request)
FLAG_PORT_RESUME:   EQU BASE_HUB0+5 ; Main loop flag for port resume control (host request)
FLAG_REMOTE_WAKEUP: EQU BASE_HUB0+6 ; Set if MASK_REMOTE_WAKEUP and Request to
                                    ; SET_FEATURE(DEVICE_REMOTE_WAKEUP)
FLAG_PORT_DISABLED: EQU BASE_HUB0+7 ; Set if Hub HW disables a port (babble)
STATE_RESUME_PENDING:EQU BASE_HUB0+8; State of port resume (DS port initiated)
SIZE_HUB_EP1_DATA:  EQU 1           ; Size of data to return in BITMAP_HUB_EP1
BITMAP_HUB_EP1:     EQU BASE_HUB0+9 ; Hub EP1 interrupt bitmap

;(SIMULATE_SUSPEND) See SUSPEND.ASM
BITMAP_ENABLED_UNSUSPENDED: EQU BASE_HUB+10 ; Enabled and unsuspended ports prior to
                                          ; hub suspend
FLAG_STALL:	    EQU BASE_HUB0+11 ; STALL flag is used in GET_STATUS request to an endpoint
BASE_HUB1:          EQU BASE_HUB0+12

;---------------------------
SIZE_HUB_STATUS:    EQU 4           ; size of status plus change
HUB_STATUS_BASE:    EQU BASE_HUB1+0
HUB_wHubStatus_LSB: EQU BASE_HUB1+0 ; (r.e. [2] Table 11-13, page 268)
                                    ; Hub Status Field LSB, bit 0=Local Power Status
                                    ;                       bit 1=Over-Current Indicator
HUB_wHubStatus_MSB: EQU BASE_HUB1+1 ; Hub Status Field MSB, ALWAYS 0
HUB_wHubChange_LSB: EQU BASE_HUB1+2 ; (r.e. [2] Table 11-14, page 269)
                                    ; Hub Change Field LSB
                                    ; x x x x x x x x
                                    ; | | | | | | | |___ Local Power Status Change (bit 0)
                                    ; | | | | | | |_____ Over-Current Indicator Change
                                    ; |_|_|_|_|_|_______ (reserved, ALWAYS 0)(bits 2-7)
HUB_wHubChange_MSB: EQU BASE_HUB1+3 ; Hub Change Field MSB, ALWAYS 0 (bits 8-15)
;---------------------------
SIZE_PORT_STATUS:     EQU 4             ; size of status plus change
HUB_PORT_STATUS_BASE: EQU BASE_HUB1+4   ; Base address of hub port status/change fields
HUB_wPortStatus0_LSB: EQU BASE_HUB1+4   ; (r.e. [2] Table 11-15, page 271)
                                        ; Port Status Field LSB
                                        ; x x x x x x x x
                                        ; | | | | | | | |___ Connect Current Status (bit 0)
                                        ; | | | | | | |_____ Port Enabled/Disabled
                                        ; | | | | | |_______ Suspend
                                        ; | | | | |_________ Over-Current Indicator
                                        ; | | | |___________ Reset
                                        ; |_|_|______________ (reserved, ALWAYS 0) (bits 5-7)
HUB_wPortStatus0_MSB: EQU BASE_HUB1+5   ; Port Status Field MSB
                                        ; x x x x x x x x
                                        ; | | | | | | | |___ Port Power (bit 8)
                                        ; | | | | | | |_____ Low Speed Device Attached
                                        ; |_|_|_|_|_|________ (reserved, ALWAYS 0) (bits 10-15)


HUB_wPortChange0_LSB: EQU BASE_HUB1+6   ; (r.e. [2] Table 11-15, page 272)
                                        ; Port Change Field LSB
                                        ; x x x x x x x x
                                        ; | | | | | | | |___ Connect Status Change (bit 0)
                                        ; | | | | | | |_____ Port Enable/Disable Change
                                        ; | | | | | |_______ Suspend Change
                                        ; | | | | |_________ Over-Current Indicator Change
                                        ; | | | |___________ Reset Change
                                        ; |_|_|______________ (reserved, ALWAYS 0) (bits 5-7)
HUB_wPortChange0_MSB: EQU BASE_HUB1+7   ; Port Change Field MSB, ALWAYS 0 (bits 8-15)
;allocate space for more ports as specified by MAX_PORTS
BASE_HUB2:            EQU HUB_PORT_STATUS_BASE+MAX_PORTS*SIZE_PORT_STATUS

;---------------------------
HUB_DATA_COUNT:       EQU BASE_HUB2+0   ; descriptor size
HUB_DATA_START:       EQU BASE_HUB2+1   ; offset to descriptor data start address
HUB_BYTE_COUNT:       EQU BASE_HUB2+2   ; SEND_BUFFER byte counter
HUB_PORT:             EQU BASE_HUB2+3   ; port number for Hub Type port specific requests
HUB_SUSPEND_COUNT:    EQU BASE_HUB2+4   ; 1ms counter for hub port suspend
HUB_PORT_RESET_COUNT: EQU BASE_HUB2+5   ; 1ms counter for port reset (host initiated)
HUB_PORT_RESUME_COUNT:EQU BASE_HUB2+6   ; 1ms counter for port resume (host initiated)
HUB_OVERCURRENT_COUNT:EQU BASE_HUB2+7   ; 1ms counter for port overcurrent
HUB_LAST_CONNECT_STAT:EQU BASE_HUB2+8   ; last port connect status detected
HUB_LAST_PORT_ENABLE: EQU BASE_HUB2+9   ; last port enable
HUB_CONNECT_CHANGE:   EQU BASE_HUB2+10   ; change bits in hub connect status
HUB_ENABLE_CHANGE:    EQU BASE_HUB2+11  ; change bits in enable status (babble)
HUB_RESET_PORT:       EQU BASE_HUB2+12  ; Port number for SET_PORT_FEATURE(PORT_RESET) req's
HUB_RESUME_PORT:      EQU BASE_HUB2+13  ; Port number for CLEAR_PORT_FEATURE(PORT_SUSPEND)
HUB_PORT_CTRL:        EQU BASE_HUB2+14  ; Port number for SET_PORT_CONTROL routine
HUB_PORT_ACTION:      EQU BASE_HUB2+15  ; Forcing action for SET_PORT_CONTROL routine
HUB_EP0_TOKEN:        EQU BASE_HUB2+16  ; Token received via Hub EP0
SHADOW_PORT_SPEED:    EQU BASE_HUB2+17  ; Shadow of REG_PORT_SPEED (INT_HUB)
SHADOW_CONNECT_STAT:  EQU BASE_HUB2+18  ; Shadow of REG_CONNECT_STAT (INT_HUB)
SHADOW_PORT_RESUME:   EQU BASE_HUB2+19  ; Shadow of REG_PORT_RESUME
HUB_RESUME_SPEED:     EQU BASE_HUB2+20	; @@@RGR
SHADOW_PORT_OC:       EQU BASE_HUB2+21  ; (BEN) added to debounce overcurrent sample
HUB_END:              EQU BASE_HUB2+22


; TIMER.ASM definitions
TIMER_BASE:         EQU HUB_END
TIMER_END:          EQU TIMER_BASE+1


; GLOBAL definitions (usable across module boundries...use with caution)
BASE_GLOBAL:        EQU TIMER_END
TEMP:               EQU BASE_GLOBAL+0   ; temporary scratch pad
TEMP1:              EQU BASE_GLOBAL+1   ; another scratch pad
TEMP2:              EQU BASE_GLOBAL+2   ; yet another
TEMP3:              EQU BASE_GLOBAL+3   ; and another
UNUSED_INTS:        EQU BASE_GLOBAL+4   ; bit mask for vectors to unused interrupts
GLOBAL_END:         EQU BASE_GLOBAL+5   ; !!!    DO NOT allocate data after this label      !!!
                                        ; !!! Used to check link file for last allocate loc !!!

; HID definitions
BASE_HID:            EQU GLOBAL_END
STATE_HID:           EQU BASE_HID
BASE_HID_END:   	EQU BASE_HID + 1
BASE_HID1:           EQU BASE_HID+1       ;(KEYBOARD) see KB.ASM
;BASE__DATA_END:     EQU BASE_KB1_END     ;(KEYBOARD) see KB.ASM

;------------------------------------------------------------------------
DSP_PRESET:         EQU D8h             ; Data Stack Pointer preset address
;(DEBUG)

; Reserved USB fifo's
FIFO_SIZE:          EQU 8

FIFO_EP_A2:         EQU E8h

FIFO_EP_A1:         EQU F0h

; Keyboard Endpoint 0, default pipe, Status and Control I/O
FIFO_EP_A0:         EQU F8h
HID_bmRequestType:   EQU F8h
HID_bRequest:        EQU F9h
HID_wValue_LSB:      EQU FAh                 ; Little-Endian format
HID_wValue_MSB:      EQU FBh
HID_wIndex_LSB:      EQU FCh
HID_wIndex_MSB:      EQU FDh
HID_wLength_LSB:     EQU FEh
HID_wLength_MSB:     EQU FFh
; Hub Endpoint 1, Status change endpoint
FIFO_EP_B1:         EQU D8h
; Hub Endpoint 0 , default pipe, Status and Control I/O
FIFO_EP_B0:         EQU E0h
HUB_bmRequestType:  EQU E0h
HUB_bRequest:       EQU E1h
HUB_wValue_LSB:     EQU E2h                 ; Little-Endian format
HUB_wValue_MSB:     EQU E3h
HUB_wIndex_LSB:     EQU E4h
HUB_wIndex_MSB:     EQU E5h
HUB_wLength_LSB:    EQU E6h
HUB_wLength_MSB:    EQU E7h