;------------------------------------------------------------------------
; RESUME.ASM
;------------------------------------------------------------------------
; Port and Hub resume control routines and TASK's
;------------------------------------------------------------------------
;------------------------------------------------------------------------
SEND_GLOBAL_RESUME:
; Send resume (K) signaling upstream (US), to all enabled (unsuspended) DS
; ports and to the port number in [A] (suspended port initiating resume, if
; applicable).
; On entry: A = suspended port, else FFh
;------------------------------------------------------------------------
    CMP     A, FFh                      ; suspended port initiating resume?
    JZ      SGR_NOT_SUSPENDED           ; if FFh, port was not suspended
SGR_SUSPENDED:
    MOV     [HUB_RESUME_PORT], A        ; save the port

; clear suspend bit in REG_PORT_SUSPEND
    INDEX   TABLE_PORT_MASK             ; get mask for the port
    CPL     A                           ; complement mask
    MOV     [TEMP], A
    read    REG_PORT_SUSPEND            ; get port suspend register contents
    AND     A, [TEMP]                   ; clear port suspend bit
    write_A REG_PORT_SUSPEND            ; save back to port suspend register
; set corresponding wPortStatus wPortChange bits and update host
    MOV     A, [HUB_RESUME_PORT]        ; get port number
    SWAP    A, X                        ; use X as offset into state base
    MOV     A, PORT_ENABLED             ; A = state of selected port
    MOV     [X+HUB_PORT_STATE_BASE], A
    MOV     A, ~MASK_PORT_SUSPEND       ; clear the suspend status bit
    MOV     [TEMP], A
    MOV     A, [HUB_RESUME_PORT]
    CALL    CLEAR_PORT_STATUS_BIT
    MOV     A, MASK_PORT_SUSPEND        ; set the suspend change bit
    MOV     [TEMP], A
    MOV     A, [HUB_RESUME_PORT]
    CALL    SET_PORT_CHANGE_BIT

SGR_NOT_SUSPENDED:
    read    REG_PORT_SUSPEND            ; read the suspend register bitmap
    CPL     A                           ; complement
    MOV     [TEMP], A                   ; save to TEMP
    read    REG_PORT_ENABLE             ; read the enable register bitmap
    AND     A, [TEMP]                   ; mask off suspended ports
    CMP     A, 0                        ; if bitmap = 0, done!
    JZ      SGR_NONE_ENABLED

SGR_PORTS_ENABLED:                      ; else some ports enabled and unsuspended
    PUSH    A                           ; save bitmap
    MOV     A, 0
    MOV     [TEMP1], A                  ; use TEMP1 as forcing action LOW
    MOV     [TEMP2], A                  ; use TEMP2 as forcing action UP
    POP     A                           ; restore bitmap
SGR_LOW_LOOP:
    CALL    PORT_FROM_BITMAP            ; A = port, X = modified bitmap
    CMP     A, 4                        ; port > 3?
    JNC     SGR_UPPER_PORT              ; if yes, an upper port?
    PUSH    A                           ; save port number
    INDEX   TABLE_PORT_MASK             ; get port mask
    MOV     [TEMP3], A                  ; save to TEMP3
    read    REG_PORT_SPEED              ; read port speed register
    AND     A, [TEMP3]                  ; AND with port mask
    JZ      SGR_LOW_FULL_SPEED          ; if 0, port is full speed
    MOV     A, 4                        ; set offset to 4
    JMP     SGR_LOW_LOW_SPEED           ; port is low speed
SGR_LOW_FULL_SPEED:
    MOV     A, 0                        ; set offset to 0
SGR_LOW_LOW_SPEED:
    MOV     [TEMP3], A                  ; save offset to TEMP3
    POP     A                           ; restore port
    ADD     A, [TEMP3]                  ; add forcing action offset
    INDEX   TABLE_FORCE_RESUME          ; get forcing action for the port
    OR      [TEMP1], A                  ; OR to final action
    MOV     A, X                        ; A = bitmap
    CMP     A, 0                        ; done?
    JZ      SGR_READY                   ; yes, set forcing actions
    JMP     SGR_LOW_LOOP                ; else test more bits

SGR_UPPER_PORT:                         ; process upper ports
    PUSH    A                           ; save port number
    INDEX   TABLE_PORT_MASK             ; get port mask
    MOV     [TEMP3], A                  ; save to TEMP3
    read    REG_PORT_SPEED              ; read port speed register
    AND     A, [TEMP3]                  ; AND with port mask
    JZ      SGR_UPPER_FULL_SPEED        ; if 0, port is full speed
    MOV     A, 4                        ; set offset to 4
    JMP     SGR_UPPER_LOW_SPEED         ; port is low speed
SGR_UPPER_FULL_SPEED:
    MOV     A, 0                        ; set offset to 0
SGR_UPPER_LOW_SPEED:
    MOV     [TEMP3], A                  ; save offset to TEMP3
    POP     A                           ; restore port
    SUB     A, 4                        ; normalize port for TABLE_FORCE_RESUME
    ADD     A, [TEMP3]                  ; add forcing action offset
    INDEX   TABLE_FORCE_RESUME          ; get forcing action for the port
    OR      [TEMP2], A                  ; OR to final action
    MOV     A, X                        ; A = bitmap
    CMP     A, 0                        ; done?
    JZ      SGR_READY                   ; yes, set forcing actions
    CALL    PORT_FROM_BITMAP            ; no, A = port, X = modified bitmap
    JMP     SGR_UPPER_PORT              ; test more bits

SGR_READY:
    MOV     A, [TEMP1]                  ; get forcing action for lower ports
    write_A REG_PORT_CTRL_LOW           ; set action
    MOV     A, [TEMP2]                  ; get forcing action for upper ports
    write_A REG_PORT_CTRL_UP            ; set action

SGR_NONE_ENABLED:
    read    REG_STAT_CTRL               ; force upstream port to resume (K)
    AND     A, ~MASK_UPSTREAM_CONTROL   ; clear current control bits
    OR      A, MASK_FORCE_UPSTREAM_K    ; set to force K
    write_A REG_STAT_CTRL               ; and save
    MOV     A, 0                        ; clear resume counter
    MOV     [HUB_PORT_RESUME_COUNT], A
SGR_WAIT:
    MOV     A, [HUB_PORT_RESUME_COUNT]  ; get counter
    CMP     A, SEND_RESUME_LIMIT        ; timeout? (counter = limit?)
    JNZ     SGR_WAIT                    ; no
    MOV     A, 0                        ; yes
    write_A REG_PORT_CTRL_LOW           ; clear port control registers
    write_A REG_PORT_CTRL_UP
    read    REG_STAT_CTRL               ; set upstream to not forcing
    AND     A, ~MASK_UPSTREAM_CONTROL   ; mask off control bits
    write_A REG_STAT_CTRL
SEND_RESUME_END:
;    clr_port0_bit MASK_BIT_6
    RET

;------------------------------------------------------------------------
SEND_LS_EOP:
; Send Low Speed End Of Packet downstream for termination of resume signaling.
; [1] 7.1.11.2 EOP Width (page 125): Transmitter EOP width for low speed
;     transmissions must be between 1.25us and 1.50us
; On Entry: [A] = port to send LS EOP
; Low-speed EOP is SE0 for 2 low-speed bit times, J for >1 low-speed bit time
;       SE0: (1/1.5M) * 2 = 1.3333us    J: (1/1.5M) = 0.667us
; Equivalent CPU cycles:
;       SE0: 1.3333us /(1/12MHz) = 16 cycles
;       J:   8 cycles
; Actual Implementation:
;       SE0: 17 cycles (1.417us)
;       J:    9 cycles (0.750us)
;------------------------------------------------------------------------

    MOV     [TEMP1], A                  ; save port to TEMP1
    PUSH    A                           ; save port
    INDEX   TABLE_PORT_MASK             ; get port mask
    MOV     [TEMP2], A                  ; save to TEMP2
    read    REG_PORT_SPEED              ; read port speed register
    AND     A, [TEMP2]                  ; AND with port mask
    JZ      LS_EOP_FULL_SPEED           ; if 0, port is full-speed
    MOV     A, 4                        ; else set offset into TABLE_LS_EOP_END = 4
    JMP     LS_EOP_LOW_SPEED
LS_EOP_FULL_SPEED:
    MOV     A, 0                        ; if full-speed, no offset for LS EOP
LS_EOP_LOW_SPEED:
    MOV     [TEMP2], A                  ; save offset to TEMP2
    POP     A                           ; restore port
    CMP     A, 4                        ; compare against 4
    JC      LS_EOP_LOWER                ; HUB_RESUME_PORT <  4
    JMP     LS_EOP_UPPER                ; HUB_RESUME_PORT >= 4

LS_EOP_LOWER:
    INDEX   TABLE_PORT_CONTROL_MASK     ; get control mask (use for not forcing!)
    MOV     [TEMP], A                   ; save to TEMP
    read    REG_PORT_CTRL_LOW           ; get port control bits
    AND     A, [TEMP]                   ; mask off control bits
    PUSH    A                           ; stack it
    MOV     [TEMP], A                   ; save to TEMP (control bits = 0)

    MOV     A, [TEMP1]                  ; get port
    ADD     A, [TEMP2]                  ; add offset if low-speed
    INDEX   TABLE_LS_EOP_END            ; get command to set J state
    OR      A, [TEMP]                   ; OR with last TEMP (control bits = 0)
    PUSH    A                           ; stack it

    MOV     A, [TEMP1]                  ; get port
    INDEX   TABLE_LS_EOP_START          ; get SE0 command for port
    MOV     [TEMP], A                   ; save to TEMP
    read    REG_PORT_CTRL_LOW           ; get current port control
    OR      A, [TEMP]                   ; set port control bits

LS_EOP_LOWER_SEND:                      ; send to a lower port
    write_A REG_PORT_CTRL_LOW           ; set port control
    NOP                                 ; 4 clocks
    NOP                                 ; 4 clocks
    POP     A                           ; 4 clocks
    write_A REG_PORT_CTRL_LOW           ; 5 clocks = 17 clocks [set port control]
                                        ; .....SE0 for 1.417us.....
    POP     A                           ; 4 clocks
    write_A REG_PORT_CTRL_LOW           ; 5 clocks = 9 clocks
                                        ; .....IDLE (J) for 0.750us.....
    JMP     SEND_LS_EOP_END             ; get out!

LS_EOP_UPPER:
    SUB     A, 4                        ; zero base the port
    MOV     [TEMP1], A                  ; save to TEMP1
    INDEX   TABLE_PORT_CONTROL_MASK     ; get control mask (use for not forcing!)
    MOV     [TEMP], A                   ; save to TEMP
    read    REG_PORT_CTRL_UP            ; get port control bits
    AND     A, [TEMP]                   ; mask off control bits
    PUSH    A                           ; stack it
    MOV     [TEMP], A                   ; save to TEMP (control bits = 0)

    MOV     A, [TEMP1]                  ; get port
    ADD     A, [TEMP2]                  ; add offset if low-speed
    INDEX   TABLE_LS_EOP_END            ; get command to set J state
    OR      A, [TEMP]                   ; OR with last TEMP (control bits = 0)
    PUSH    A                           ; stack it

    MOV     A, [TEMP1]                  ; get zero base port
    INDEX   TABLE_LS_EOP_START          ; get SE0 command for port
    MOV     [TEMP], A                   ; save to TEMP
    read    REG_PORT_CTRL_UP            ; get current port control
    OR      A, [TEMP]                   ; set port control bits

LS_EOP_UPPER_SEND:                      ; send to an upper port
    write_A REG_PORT_CTRL_UP            ; set port control
    NOP                                 ; 4 clocks
    NOP                                 ; 4 clocks
    POP     A                           ; 4 clocks
    write_A REG_PORT_CTRL_UP            ; 5 clocks = 17 clocks [set port control]
    POP     A                           ; 4 clocks
    write_A REG_PORT_CTRL_UP            ; 5 clocks = 9 clocks

SEND_LS_EOP_END:
    RET

;------------------------------------------------------------------------
SEND_RESUME:
; Send resume (K) signaling downstream. During resume signaling, the port
; enable and suspend bits are cleared. The port it tested for full or low
; speed and resume (K) signaling is then forced. TASK_PORT_RESUME will
; terminate the resume signaling by sending the LS EOP and reenabling the port.
; Note: if the port is not disabled prior to forcing resume, the potential
; exists for babble detection by the hub hardware and subsequent disabling
; of the port by the hub.
; On entry A = port to send resume (0-3)
;------------------------------------------------------------------------
    MOV     [HUB_RESUME_PORT], A        ; save to hub resume port
    MOV     [HUB_PORT_CTRL], A          ; save port to HUB_PORT_CTRL

    CMP     A,MAX_EMBEDDED_PORTS
    JNC      SKIP_HW
; clear enable and suspend bits for HUB_RESUME_PORT
    INDEX   TABLE_PORT_MASK             ; get mask for the port
    CPL     A                           ; complement mask
    MOV     [TEMP], A                   ; save to TEMP

;(RESUME_FIX)CATC Test 4*********************
    read    REG_PORT_ENABLE             ; get the enable register contents
    AND     A, [TEMP]                   ; clear the enable bit
    write_A REG_PORT_ENABLE             ; save back to port enable register
    MOV     [HUB_LAST_PORT_ENABLE], A   ; save to last port enable
;********************************************

    read    REG_PORT_SUSPEND            ; get port suspend register contents
    AND     A, [TEMP]                   ; clear port suspend bit
    write_A REG_PORT_SUSPEND            ; save back to port suspend register

; Force RESUME (K) signaling (Full-speed=Differential 0, Low-speed=Differential 1)
    MOV     A, [TEMP]                   ; get complemented port mask
    CPL     A                           ; complement to true
    MOV     [TEMP], A                   ; save back to TEMP
    read    REG_PORT_SPEED              ; get port speed register
    AND     A, [TEMP]                   ; AND with mask
    MOV	    [HUB_RESUME_SPEED], A	; @@@RGR
    JZ      SR_FULL_SPEED               ; if 0, is full speed device
    MOV	    A, [TEMP]			; @@@RGR
    CPL	    A				; @@@RGR
    MOV	    [TEMP], A			; @@@RGR
    read    REG_PORT_SPEED		; @@@RGR
    AND	    A, [TEMP]			; @@@RGR
    write_A REG_PORT_SPEED		; @@@RGR
    MOV	    A, [TEMP]			; @@@RGR
    CPL	    A				; @@@RGR
    MOV	    [TEMP], A			; @@@RGR
    MOV     A, MASK_FORCE_LS_K          ; else is low speed device (force low-speed K)
    JMP     SR_LOW_SPEED
SR_FULL_SPEED:
    MOV     A, MASK_FORCE_FS_K          ; force full-speed K (resume) state
SR_LOW_SPEED:
    MOV     [HUB_PORT_ACTION], A        ; save forcing action
    CALL    SET_PORT_CONTROL            ; set action to appropriate port
SKIP_HW:
    MOV     A, 0
    MOV     [HUB_PORT_RESUME_COUNT], A  ; clear the count, 1.024ms timer increments this count
    set_flag    FLAG_PORT_RESUME        ; set the main loop flag for continued(time delay)processing
    RET

;------------------------------------------------------------------------
TASK_RESUME_PENDING:
; This is a STATE (not FLAG) driven task (main loop task) (STATE_RESUME_PENDING)
; This task services a suspended port that is initiating a resume event
; (Port initiated SELECTIVE RESUME to an AWAKE HUB).
; INT_HUB initiates the resume event by setting the STATE to RESUME_START.
; Hub hardware forcing of resume (K) is overriden by this routine then
; uses TASK_PORT_RESUME for continued sequencing (if not in use by SET_PORT_RESUME).
;------------------------------------------------------------------------

    test_flag   FLAG_PORT_RESUME        ; test host initiated resume flag
    JZ      TRP_WAIT                    ; (want to use TASK_PORT_RESUME)
                                        ; (wait if in use)
    MOV     A, [SHADOW_PORT_RESUME]     ; get hub port resume bitmap
    CALL    PORT_FROM_BITMAP            ; get port from bitmap (A = port, X = bitmap)
    SWAP    A, X                        ; A = bitmap, X = port
    MOV     [SHADOW_PORT_RESUME], A     ; save modified bitmap
    SWAP    A, X                        ; A = port
    CALL    SEND_RESUME

    MOV     A, RESUME_OVERRIDE
    MOV     [STATE_RESUME_PENDING], A   ; bump resume pending state to override
TRP_WAIT:
    RET

;------------------------------------------------------------------------
TASK_PORT_RESUME:
; USB Downstream Port Resume control task (main loop task)
; This task is initiated by CLEAR_PORT_SUSPEND (HUB_EP0C.ASM)
; or TASK_RESUME_PENDING (this module RESUME.ASM).
; This task handles the timeout for RESUME (K) assertion and the removal of the
; Resume (K) followed by Low-Speed EOP, and then places the port back into the
; enabled state.
; wPortStatus and wPortChange are handled by this routine.
; By spec ([1],[2]) the hub must wait 3ms after LSEOP to report the status
; change to the host (TPR_WAIT and TPR_COMPLETE).
;------------------------------------------------------------------------
    MOV     A, [HUB_PORT_RESUME_COUNT]  ; get timer incremented counter
    CMP     A, PORT_RESUME_LIMIT        ; expired?
    JNZ     TPR_WAIT                    ; no, test for end of resume signaling

; resume timeout expired, remove (K) signaling and send LS EOP
TPR_RESUME_TIMEOUT:
    INC     [HUB_PORT_RESUME_COUNT]     ; bump counter to prevent sending
                                        ; LSEOP's for the next 1ms
    MOV     A, [HUB_RESUME_PORT]        ; get port
    CMP     A,MAX_EMBEDDED_PORTS
    JNC      TASK_PORT_RESUME_END
   CALL    SEND_LS_EOP                 ; send Low-Speed EOP to appropriate port

;(RESUME_FIX)CATC Test 4*********************
;reenable the port receiving resume signaling
    IORD    REG_PORT_SPEED		; @@@RGR
    OR	    A, [HUB_RESUME_SPEED]	; @@@RGR
    IOWR    REG_PORT_SPEED		; @@@RGR
    MOV     A, [HUB_RESUME_PORT]        ; get port
    INDEX   TABLE_PORT_MASK             ; get mask for the port
    MOV     [TEMP], A
    read    REG_PORT_ENABLE             ; get port enable register contents
    OR      A, [TEMP]                   ; set port enable bit
    write_A REG_PORT_ENABLE             ; save back to port enable register
    MOV     [HUB_LAST_PORT_ENABLE], A   ; save to last port enable
;********************************************


    JMP     TASK_PORT_RESUME_END        ; get out

; test for resume sequence complete
TPR_WAIT:                               ; at least 3 ms after LS-EOP sent
    CMP     A, PORT_RESUME_COMPLETE     ; completely done?
    JNZ     TASK_PORT_RESUME_END        ; if !=, not done yet!

TPR_COMPLETE:
; set corresponding wPortStatus wPortChange bits and update host
    MOV     A, [HUB_RESUME_PORT]        ; get port number
    SWAP    A, X                        ; use X as offset into state base
    MOV     A, PORT_ENABLED             ; A = state of selected port
    MOV     [X+HUB_PORT_STATE_BASE], A

    MOV     A, ~MASK_PORT_SUSPEND       ; clear the suspend status bit
    MOV     [TEMP], A
    MOV     A, [HUB_RESUME_PORT]
    CALL    CLEAR_PORT_STATUS_BIT

    MOV     A, MASK_PORT_SUSPEND        ; set the suspend change bit
    MOV     [TEMP], A
    MOV     A, [HUB_RESUME_PORT]
    CALL    SET_PORT_CHANGE_BIT

    clr_flag    FLAG_PORT_RESUME        ; clear the main loop flag to discontinue processing
    test_state  STATE_RESUME_PENDING, RESUME_START ; has a port initiated a resume
                                        ; while host resume in progress?
    JZ      TASK_PORT_RESUME_END        ; port initiated resume is pending (don't clear)
    MOV     A, RESUME_COMPLETE          ; clear STATE_RESUME_PENDING if = OVERRIDE
    MOV     [STATE_RESUME_PENDING], A
TASK_PORT_RESUME_END:
    RET

;------------------------------------------------------------------------
; ROM Tables
;------------------------------------------------------------------------
    XPAGEOFF

TABLE_FORCE_RESUME:
; Full-speed K signaling (differential 0)
    DB      00000010b       ; force resume (K) ports 0 or 5  (full-speed)
    DB      00001000b       ;                        1    6
    DB      00100000b       ;                        2    7
    DB      10000000b       ;                        3
; Low-speed K signaling (differential 1)
    DB      00000001b       ; force resume (K) ports 0 or 5  (low-speed)
    DB      00000100b       ;                        1    6
    DB      00010000b       ;                        2    7
    DB      01000000b       ;                        3

    XPAGEON